#include "cmsis_os.h"
#include "stm32f4xx_hal.h"
#include "main.h"
#include "Http.h"
#include "Wakeup.h"
#include "lwip/sockets.h"
#include <string.h>
#include "lwip.h"

osThreadId httpdTaskHandle;
MAC_t target = {0x3C, 0x97, 0x0E, 0x8E, 0x15, 0x2C};

static void TestWOL(int socket, const char* hdr, const char* Data);
static void InputForm(int socket, const char* hdr, const char* Data);
static void SaveForm(int socket, const char* hdr, const char* Data);

static void ReadEEPROM();
static void WriteEEPROM();

const HTTP_File_t HTTP_Files[] = {
  { .Filename = "/"    , .Method = GET,  .Handler = InputForm  , .Data = ""},
  { .Filename = "/save", .Method = POST, .Handler = SaveForm   , .Data = ""},
  { .Filename = "/test", .Method = GET,  .Handler = TestWOL    , .Data = ""},
  { .Filename = ""     , .Method = GET,  .Handler = NULL       , .Data = ""}
};

//Main task to handle button press
void MainTask(const void* arguments){

  //Read stored config
  ReadEEPROM();

  //Initialize LWIP stack
  MX_LWIP_Init();

  //Send WOL packet when button is pressed
  while(1){
    if(HAL_GPIO_ReadPin(WKUP_GPIO_Port, WKUP_Pin) == 1){
      HAL_GPIO_WritePin(LD4_GPIO_Port, LD4_Pin, GPIO_PIN_SET);
      WAKEUP_Send(target, 9);
    }
    else
      HAL_GPIO_WritePin(LD4_GPIO_Port, LD4_Pin, GPIO_PIN_RESET);
    osDelay(200);
    printf("[APP] Loop!\r\n");
  }    
}

static void TestWOL(int socket, const char* hdr, const char* Data){
  if( WAKEUP_Send(target, 9) < 0)
    HTTP_Static(socket, hdr, "Error while sending packet!");
  else
    HTTP_Static(socket, hdr, "Packet was sent");
}

//EEPROM access code
//Save in FLASH ?
MAC_t* flash_target = (MAC_t*) ADDR_FLASH_SECTOR_11;

static void WriteEEPROM(){
  HAL_FLASH_Unlock();

  __HAL_FLASH_CLEAR_FLAG(FLASH_FLAG_EOP | FLASH_FLAG_OPERR | FLASH_FLAG_WRPERR | FLASH_FLAG_PGAERR | FLASH_FLAG_PGPERR | FLASH_FLAG_PGSERR);

  FLASH_Erase_Sector(FLASH_SECTOR_11, VOLTAGE_RANGE_3);

  //Write data into flash
  for(int i = 0; i< 8; i++){
    HAL_FLASH_Program( FLASH_TYPEPROGRAM_BYTE, (uint32_t) flash_target + i, target[i]);
  }  

  HAL_FLASH_Lock();

  ReadEEPROM();
}

static void ReadEEPROM(){
  //Read it back
  memcpy(&target, flash_target, sizeof(target));
}


// HTTP pages handlers below this
const char Form1[] = "HTTP/1.1 200 OK\r\n\
\r\n\
<html>\
  <head>\
    <meta http-equiv='Content-type' content='text/html; charset=ascii'>\
  </head>\
  <body>\
    <h1>Wake-on-LAN sender</h1>\
      <form action='/save' method='POST'>\
        <table>\
          <tr><td>Target MAC:</td><td>\
            <input type='text' name='mac' value='%02x:%02x:%02x:%02x:%02x:%02x'/></td></tr>\
          <tr><td colspan='2'><input type='submit' text='Save' name='submit' value='submit'/></td></tr>\
        </table>\
      </form>\
      <a href='/test'>Test the settings</a>  \
    </body>\
</html>";

char PageBuffer[1500];
static void InputForm(int socket, const char* hdr, const char* Data){
  sprintf(PageBuffer, Form1, target[0],
                             target[1],
                             target[2],
                             target[3],
                             target[4],
                             target[5]);
  write(socket, PageBuffer, strlen(PageBuffer));
}

const char Result[] = "HTTP/1.1 200 OK\r\n\
\r\n\
<html>\
  <body>\
    <h1>Wake-on-LAN sender</h1>\
      Mac was setted to %02x:%02x:%02x:%02x:%02x:%02x</br>\
      <a href='/'>Back</a>  \
    </body>\
</html>";

static void SaveForm(int socket, const char* hdr, const char* Data){
  int buffer_ptr = 0;
  int ptr_increment = 1;
 
  memset(PageBuffer, 0, sizeof(PageBuffer));
 
  while((strstr(PageBuffer, "submit=submit") == NULL)){
    ptr_increment = read(socket, &PageBuffer[buffer_ptr], sizeof(PageBuffer) - buffer_ptr);
    if(ptr_increment <= 0) return;
    buffer_ptr += ptr_increment;
    if(buffer_ptr >= sizeof(PageBuffer)) return;
  }

  int match = sscanf(PageBuffer, "mac=%2hhx%%3A%2hhx%%3A%2hhx%%3A%2hhx%%3A%2hhx%%3A%2hhx%%3A",
         &target[0],
         &target[1],
         &target[2],
         &target[3],
         &target[4],
         &target[5]);

  printf("[APP] Scanf MAC address match %d\r\n", match);

  //Save data into flash
  WriteEEPROM();

  printf(PageBuffer);

  memset(PageBuffer, 0, sizeof(PageBuffer));
  sprintf(PageBuffer, Result, target[0],
                              target[1],
                              target[2],
                              target[3],
                              target[4],
                              target[5]);
  write(socket, PageBuffer, strlen(PageBuffer));
}
