/*
  Very simple HTTP server 
  Cafee
*/

#include <string.h>
#include <strings.h>
#include "lwip/sockets.h"
#include "Http.h"
#include "cmsis_os.h"

extern HTTP_File_t HTTP_Files[];
static const char NotFound1[] = "HTTP/1.1 404 NOT FOUND\r\n\r\n<html><body><h1> File ";
static const char NotFound2[] = " was not found!</h1></body></html>";
static char buffer[HTTP_BUFFER_SIZE];

//Simplest HTTP handler
void HTTP_Static(int client_sck, const char* headers, const char* Data){
  const char OK[] = "HTTP/1.1 200 OK\r\n\r\n";
  write(client_sck, OK , strlen(OK));
  write(client_sck, Data, strlen(Data));
}

//Serve incoming request
void HTTP_Serve(int client_sck){
  int buf_ptr = 0, read_cnt;

  //Get whole request from client
  while(strstr(buffer, "\r\n\r\n") == NULL){
    read_cnt = read(client_sck, &buffer[buf_ptr], sizeof(buffer)-buf_ptr);
    if(read_cnt <= 0) return;   // read_cnd == 0 means EOF and we will throw request out
    buf_ptr += read_cnt;
    if(buf_ptr >= sizeof(buffer)){
      close(client_sck);
      return;
    }
  } 

  char* method_str = strtok(buffer, " ");
  char* filename = strtok(NULL, " ");
  strtok(NULL, "\r\n"); //Ignore everything to EOL
  char* headers = strtok(NULL, "\0");

  HTTP_Method_t method;
  if(strcasecmp(method_str, "GET") == 0)
    method = GET;
  else if (strcasecmp(method_str, "POST") == 0)
    method = POST;
  else if (strcasecmp(method_str, "PUT") == 0)
    method = PUT;
  else if (strcasecmp(method_str, "DELETE") == 0)
    method = DELETE;
  else{
    printf("[HTTP] Unknown method %s\r\n", method_str);
    close(client_sck);
    return;
  }

  //Now we have whole request in buffer to process
  printf("[HTTP] Method: %s, File %s\r\n", method_str, filename);

  //Find file in structure
  int file_id = 0;

  while(HTTP_Files[file_id].Handler != NULL){
    if(strcasecmp(HTTP_Files[file_id].Filename, filename) == 0 &&
      HTTP_Files[file_id].Method == method){
      printf("[HTTP] Handle file...\r\n");
      //Execute file's handler
      HTTP_Files[file_id].Handler(client_sck, headers, HTTP_Files[file_id].Data);
      close(client_sck);
      printf("[HTTP] DONE\r\n");
      return;
    }
    file_id++;
  }

  //No file found
  write(client_sck, NotFound1, sizeof(NotFound1));
  write(client_sck, filename, strlen(filename));
  write(client_sck, NotFound2, sizeof(NotFound2));

  close(client_sck);
  printf("[HTTP] File %s not fount\r\n", filename);
}

void HTTP_Thread(void const* arguments){
  int listen_sck;
  struct sockaddr_in listen_addr;

  osDelay(500); //Wait to initialize LWIP stack

  //Create socket
  listen_sck = socket(AF_INET, SOCK_STREAM, 0);
  if(listen_sck < 0){
    printf("[HTTP] Can't create socket\r\n");
    return;
  }

  //Bind to ip and port (0.0.0.0:HTTP_PORT)
  listen_addr.sin_family = AF_INET;
  listen_addr.sin_port = htons(HTTP_PORT);
  listen_addr.sin_addr.s_addr = INADDR_ANY; 
  if(bind(listen_sck, (struct sockaddr*)&listen_addr, sizeof(listen_addr))){
    printf("[HTTP] Can't bind to address\r\n");
    return;  
  }

  //Start listening for incoming connections
  listen(listen_sck, 5);

  printf("[HTTP] Start listening\r\n");

  while(1){
    int client_sck, size = 0;
    struct sockaddr_in remote_addr;
    //When connection arrives accept it
    client_sck = accept(listen_sck, (struct sockaddr*)&remote_addr, (socklen_t*)&size);
    printf("[HTTP] New client\r\n");
    HTTP_Serve(client_sck);
    osThreadYield();
    printf("[HTTP] Loop\r\n");
  }
}
